package com.example.julian.endlessroll.main;

import android.app.Activity;
import android.app.ActivityManager;
import android.content.Context;
import android.content.pm.ConfigurationInfo;
import android.content.res.Configuration;
import android.graphics.Typeface;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Window;
import android.view.WindowManager;
import android.widget.RelativeLayout;

import com.example.julian.endlessroll.levels.Level;
import com.example.julian.endlessroll.levels.LevelManager;
import com.example.julian.endlessroll.levels.LevelPack;
import com.example.julian.endlessroll.main.game.User;
import com.example.julian.endlessroll.main.screens.GameScreen;
import com.example.julian.endlessroll.main.screens.LevelsScreen;
import com.example.julian.endlessroll.main.screens.Screen;
import com.example.julian.endlessroll.main.screens.ScreenFlipper;
import com.example.julian.endlessroll.main.screens.StartScreen;
import com.example.julian.endlessroll.main.screens.ToolShopScreen;
import com.example.julian.endlessroll.main.screens.WorldsScreen;
import com.example.julian.endlessroll.rendering.renderer.GameRenderer;
import com.example.julian.endlessroll.sounds.SoundManager;
import com.example.julian.endlessroll.views.LevelupMessage;
import com.example.julian.endlessroll.views.TopBarData;

/**
 * Created by Julian on 06.02.2016.
 */
public class GameActivity extends Activity implements ExceptionHandler, User.LvUpListener {

    private DataStorageHandler dataStorageHandler;
    private LevelManager levelManager;
    private SoundManager soundManager;
    private User user;

    private MyGlSurfaceView glSurfaceView;
    private ScreenFlipper flipper;
    private StartScreen startScreen;
    private WorldsScreen worldsScreen;
    private LevelsScreen levelsScreen;
    private ToolShopScreen toolShopScreen;
    private GameScreen gameScreen;

    private LevelupMessage levelupMessage;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        try {
            GameLog.d("OnCreate");
            super.onCreate(savedInstanceState);
            super.requestWindowFeature(Window.FEATURE_NO_TITLE);
            super.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
                    WindowManager.LayoutParams.FLAG_FULLSCREEN);
            if (!hasGLES20())
                throw new Exception("OpenGL ES 2.0 not supported");

            dataStorageHandler = new DataStorageHandler(this);
            dataStorageHandler.readBoughtTools();
            user = dataStorageHandler.readUserData(this);
            soundManager = new SoundManager(this);
            soundManager.setSoundOn(dataStorageHandler.readIsSoundOn());
            soundManager.backgroundMusic.getPlayer().setLooping(true);
            soundManager.backgroundMusic.start();
            levelManager = new LevelManager(this, dataStorageHandler);

            this.glSurfaceView = new MyGlSurfaceView(this, new GameRenderer(this));
            Typeface typeface = Typeface.createFromAsset(getAssets(), "fontBaron.ttf");
            TopBarData topBarData = new TopBarData(this, dataStorageHandler, soundManager, levelManager, user, typeface);
            startScreen = new StartScreen(this, glSurfaceView, soundManager, levelManager, user, typeface);
            worldsScreen = new WorldsScreen(topBarData);
            levelsScreen = new LevelsScreen(topBarData);
            toolShopScreen = new ToolShopScreen(topBarData);
            gameScreen = new GameScreen(topBarData, glSurfaceView);

            levelupMessage = new LevelupMessage(this, typeface, user);

            flipper = new ScreenFlipper(this, startScreen, worldsScreen, levelsScreen, gameScreen, toolShopScreen);
            RelativeLayout relativeLayout = new RelativeLayout(this);
            relativeLayout.addView(glSurfaceView);
            relativeLayout.addView(flipper);
            relativeLayout.addView(levelupMessage.getLayout());
            //TODO: add Tutorial
            //TODO: show Energy on levelButton if collected
            //TODO: Scroll up
            //TODO: Goal animation!
            //TODO: Goal screen animation!
            setContentView(relativeLayout);
        } catch (Exception e) {
            onException(e);
            return;
        }
    }

    public void flipToScreen(final Screen.ScreenType screen) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                flipper.showScreen(screen);
            }
        });
    }

    public void setToolshopCaller(Screen.ScreenType caller) {
        toolShopScreen.setCaller(caller);
    }

    public Screen.ScreenType getCurrentScreenType() {
        return flipper.getCurrentScreen().getType();
    }

    public void selectWorld(LevelPack levelPack) {
        levelsScreen.createList(levelPack);
    }

    public void startGame(final LevelPack levelPack, final Level level) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                flipper.showScreen(Screen.ScreenType.GAME);
                gameScreen.startGame(levelPack, level);
            }
        });
    }

    @Override
    public void onLvUp(final int level) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                levelupMessage.show(level);
            }
        });
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        try {
            if (keyCode == KeyEvent.KEYCODE_BACK) {
                flipper.getCurrentScreen().onBackKeyDown();
                return true;
            }
        } catch (Exception e) {
            onException(e);
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public void onException(Exception e) {
        GameLog.e(e);
        super.finish();
    }

    @Override
    protected void onPause() {
        GameLog.d("OnPause");
        glSurfaceView.onPause();
        if (flipper.getCurrentScreen() == gameScreen)
            gameScreen.onPause();
        soundManager.pause();
        super.onPause();
    }

    @Override
    protected void onResume() {
        GameLog.d("OnResume");
        glSurfaceView.onResume();
        soundManager.resume();
        super.onResume();
    }

    @Override
    protected void onDestroy() {
        GameLog.d("OnDestroy");
        soundManager.destroy();

        dataStorageHandler.writeSoundOn(soundManager.isSoundOn());
        super.onDestroy();
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
    }

    private boolean hasGLES20() {
        ActivityManager am = (ActivityManager) getSystemService(Context.ACTIVITY_SERVICE);
        ConfigurationInfo info = am.getDeviceConfigurationInfo();
        return info.reqGlEsVersion >= 0x20000;
    }
}
