package com.example.julian.endlessroll.rendering.renderer;

import android.content.Context;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;

import com.example.julian.endlessroll.entities.Entity;
import com.example.julian.endlessroll.entities.textures.TexturePack;
import com.example.julian.endlessroll.entities.tileLists.Tile;
import com.example.julian.endlessroll.entities.tileLists.TileList;
import com.example.julian.endlessroll.main.GameLog;
import com.example.julian.endlessroll.main.game.Scene;
import com.example.julian.endlessroll.main.game.Timer;
import com.example.julian.endlessroll.rendering.MatrixCreator;
import com.example.julian.endlessroll.rendering.Quad;
import com.example.julian.endlessroll.rendering.Rendering;
import com.example.julian.endlessroll.rendering.shader.EntityShader;
import com.example.julian.endlessroll.rendering.shader.SimpleShader;
import com.example.julian.endlessroll.rendering.shader.TerrainShader;

import java.util.ArrayList;
import java.util.List;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

/**
 * Created by Julian on 22.11.2015.
 */
public class GameRenderer implements GLSurfaceView.Renderer {

    private List<Rendering> renderTargets = new ArrayList<>();
    private Rendering currentRendering;
    private MatrixCreator matrixCreator;

    private Context context;
    private Quad quad;
    //GL Context
    private ParticleRenderer particleRenderer;

    private EntityShader entityShader;
    private TerrainShader terrainShader;
    private SimpleShader simpleShader;

    private TexturePack texturePack;
    private Timer timer;

    public GameRenderer(Context context) {
        this.context = context;
        matrixCreator = new MatrixCreator();
        quad = new Quad();
        particleRenderer = new ParticleRenderer(context, quad, matrixCreator);
    }

    public void addRendering(Rendering rendering) {
        renderTargets.add(rendering);
    }

    public void setCurrentRendering(Rendering currentRendering) {
        this.currentRendering = currentRendering;
    }

    @Override
    public void onSurfaceCreated(GL10 gl, EGLConfig config) {
        GameLog.d("onSurfaceCreated");
        GLES20.glClearColor(1, 0, 0, 1.0f);
        gl.glEnable(GL10.GL_BLEND);
        gl.glBlendFunc(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
        try {
            particleRenderer.createContext();
            entityShader = new EntityShader(context);
            terrainShader = new TerrainShader(context);
            simpleShader = new SimpleShader(context);
            texturePack = new TexturePack(context);
            timer = new Timer();
        } catch (Exception e) {
            for (Rendering rendering : renderTargets)
                rendering.onException(e);
        }
        for (Rendering rendering : renderTargets) {
            rendering.initiate(texturePack, timer);
        }
    }

    @Override
    public void onSurfaceChanged(GL10 gl, int width, int height) {
        GameLog.d("onSurfaceChanged: width=" + width + ", height=" + height);
        particleRenderer.createFbo(width, height);
        GLES20.glViewport(0, 0, width, height);
        matrixCreator.setMVPMSize(width, height);
        for (Rendering rendering : renderTargets)
            rendering.setScreenSize(width, height);
    }

    @Override
    public void onDrawFrame(GL10 gl) {
        timer.update();
        if (currentRendering != null) {
            currentRendering.update();
            Scene scene = currentRendering.getScene();

            particleRenderer.renderParticlesToFbo(gl, scene);

            GLES20.glClearColor(1, 1, 1, 1.0f);
            GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);

            renderEntities(gl, scene);
            renderTileList(gl, scene.getTerrain(), scene);
            renderTileList(gl, scene.getCeiling(), scene);
            renderFbo(gl);
        }
    }

    private void renderEntities(GL10 gl, Scene scene) {
        entityShader.start();
        entityShader.loadMVPMatrix(matrixCreator, scene.cameraX);
        synchronized (scene.getBackground()) {
            for (Entity backgroundPart : scene.getBackground())
                renderEntity(gl, backgroundPart);
        }
        synchronized (scene) {
            for (Entity entity : scene)
                renderEntity(gl, entity);
        }
        entityShader.stop();
    }

    private void renderEntity(GL10 gl, Entity entity) {
        gl.glActiveTexture(GL10.GL_TEXTURE0);
        gl.glBindTexture(GL10.GL_TEXTURE_2D, entity.getTexture().getId());
        entityShader.loadTransformationMatrix(matrixCreator, entity);
        entityShader.loadAlpha(entity.getAlpha());
        entityShader.loadTextureAtlasInfos(entity.getTexture(), entity.getTextureAtlasIndex());
        quad.draw();
    }

    private void renderTileList(GL10 gl, TileList tileList, Scene scene) {
        terrainShader.start();
        terrainShader.loadMVPMatrix(matrixCreator, scene.cameraX);
        synchronized (tileList) {
            for (Tile tile : tileList) {
                gl.glActiveTexture(GL10.GL_TEXTURE0);
                gl.glBindTexture(GL10.GL_TEXTURE_2D, tile.getTexture().getId());
                terrainShader.loadTransformationMatrix(matrixCreator, tile);
                quad.draw();
            }
        }
        terrainShader.stop();
    }

    private void renderFbo(GL10 gl) {
        simpleShader.start();
        gl.glActiveTexture(GL10.GL_TEXTURE0);
        gl.glBindTexture(GL10.GL_TEXTURE_2D, particleRenderer.getFboTexture());
        quad.draw();
        simpleShader.stop();
    }


}
